// Sample application of fourier library functions
// Bala Amavasai (bpa@amavasai.org)
// Thu Oct 18 12:38:26 BST 2001
// Jan Wedekind (jan@wedesoft.de)
// Mon Aug 23 14:37:05 UTC 2004

#include <algorithm>
#include <cmath>
#include <complex>
#include <iostream>
#include <fstream>
#include "fourier.h"
#include "image.h"
#include "image_fileinput.h"
#include "image_funcs.h"
#include "image_mesaoutput.h"
#include "multi_array_op.h"

using namespace boost;
using namespace std;
using namespace mimas;

template<
  typename T,
  template< typename > class C
>
struct _creal: public unary_function< C< T >, T >
{
  T operator()( const C< T > &x ) const { return x.real(); }
};

template <
  typename T, size_t NumDims,
  template< class, size_t > class MultiArray,
  template< typename > class C
>
boost::multi_array< T, NumDims > creal
  ( const MultiArray< C< T >, NumDims > &a )
{
  return multi_func< T >( a, _creal< T, C >() );
};

int main(int argc, char **argv)
{
  int retVal = 0;
  try {
    x11_display display;
    image< int > img;
    image_mesaoutput< unsigned char >
      window1( &display ),
      window2( &display ),
      window3( &display ),
      window4( &display );

    ifstream file( "../images/stills/jellybeans-256x256.pgm", ios::binary );
    file >> img;

    const_multi_array_ref< int, 2 > data
      ( img.rawData(),
        extents[ img.getHeight() ][ img.getWidth() ] );

    {
      // Real fourier transform.
      multi_array< std::complex< double >,2 > imageFFT
	( rfft( multi_cast< double >( data ) ) );
      multi_array< double, 2 > imageLogFFT
        ( logarithm( norm< double >( imageFFT ) + 1.0 ) );

      // Remove peak caused by constant component of input image.
      imageLogFFT[ 0 ][ 0 ] = 0.0;

      // Display.
      const_image_ref< double > result( imageLogFFT.data(),
                                        imageLogFFT.shape()[1],
                                        imageLogFFT.shape()[0] );
      window1 << normalise( result, 0.0, 255.0 );

      // Inverse fourier transform.
      boost::multi_array< double, 2 >
        result2( invrfft( imageFFT ) );
      window2 << const_image_ref< double >( result2.data(),
                                            result2.shape()[1],
                                            result2.shape()[0] );
    };
    {
      // Fourier transform.
      multi_array< std::complex< double >,2 > imageFFT
	( fft( multi_cast< std::complex< double > >( data ) ) );
      multi_array< double, 2 > imageLogFFT
        ( logarithm( norm< double >( imageFFT ) + 1.0 ) );

      // Remove peak caused by constant component of input image.
      imageLogFFT[ 0 ][ 0 ] = 0.0;

      // Display.
      const_image_ref< double > result( imageLogFFT.data(),
                                        imageLogFFT.shape()[1],
                                        imageLogFFT.shape()[0] );
      window3 << normalise( result, 0.0, 255.0 );

      // Inverse fourier transform.
      boost::multi_array< double, 2 >
        result2 ( creal( invfft( imageFFT ) ) );
      window4 << const_image_ref< double >( result2.data(),
                                            result2.shape()[1],
                                            result2.shape()[0] )
              << mimas::pause();
    };
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}
