#include <iostream>
#include <cmath>
#include <fstream>
#include <valarray>
#include "multi_array_op.h"
#include "image_fileinput.h"
#include "image_mesaoutput.h"
#include "image.h"

#include "registration.h"

using namespace boost;
using namespace std;
using namespace mimas;

/* the executable in 'registration' takes the two test images
   in parameters and the size of the square area two be used for
   registration.
   Dennis Tessier Sep 2004 */
int main(int argc, char **argv)
{
  int retVal = 0;
  try {
    
    MMERROR( argc == 4, mimasexception, ,
             "Syntax: " << argv[0] << " image1 image2 size" );

    x11_display display;
    image_mesaoutput< unsigned char > window( &display );
    
    image< int > inimage1, inimage2;

    ifstream
      infile1( argv[1], ios::binary ),
      infile2( argv[2], ios::binary );
  
    MMERROR( infile1 >> inimage1, mimasexception, ,
	     "Error loading image file '" << argv[1] << '\'' );
    MMERROR( infile2 >> inimage2, mimasexception, ,
	     "Error loading image file '" << argv[2] << '\'' );

    int a = atoi(argv[3]);
    MMERROR( a > 0, mimasexception, ,
	     "3th parameter must be an integer greater than zero." );

    typedef multi_array< double, 2 > array_type;
    typedef array_type::index_range range;
    image< int > im1, im2;
    im1.init( a, a ), im2.init( a, a );

    boost::const_multi_array_ref< int, 2 >
      in1Data( inimage1.rawData(),
               extents[ inimage1.getHeight() ]
                      [ inimage1.getWidth() ] ),
      in2Data( inimage2.rawData(),
               extents[ inimage1.getHeight() ]
                      [ inimage1.getWidth() ] );
    multi_array_ref< int, 2 >( im1.rawData(),
                               boost::extents[ a ][ a ] ) =
      in1Data[ indices[ range( 0, a ) ][ range( 0, a ) ] ];
    multi_array_ref< int, 2 >( im2.rawData(),
                               boost::extents[ a ][ a ] ) =
      in2Data[ indices[ range( 0, a ) ][ range( 0, a ) ] ];

    window << registration( im1, im2 ) << mimas::pause();

  } catch ( exception &e ) {

    cerr << e.what() << endl;
    retVal = 1;

  };

  return retVal;
}
