template<typename T, template <typename, typename> class comparison>
void
correlation_tool<T, comparison>::set_rectangular_roi(point top_left, int width, int height)
{ 
  //when tracking the method is called multiple times thus the need of clearing the roi from 
  //previous call to this function.
  roi.clear();
  for(int x = 0; x < width; x++)
    for(int y = 0; y < height; y++)
      {
	roi.push_back(top_left + point(x,y));
      }
}

template<typename T, template <typename, typename> class comparison>
image<T> 
correlation_tool<T,comparison>::get_image_pattern(T color_of_empty_region)
{
  image<T> result;
  
  typename std::vector<T>::iterator values_it = pattern_.values.begin();
  std::vector<point>::iterator shape_it = pattern_.shape.begin();

  point upper_left_corner = *shape_it;
  shape_it++;
  point lower_right_corner = *shape_it;
  shape_it++;

  result.init(lower_right_corner.x - upper_left_corner.x, lower_right_corner.y - upper_left_corner.y);
  
  result.fill(color_of_empty_region);
  for(;shape_it != pattern_.shape.end(); shape_it++,values_it++)
    {
      result.setPixel(shape_it->x - upper_left_corner.x, shape_it->y - upper_left_corner.y ,*values_it);
    }

  return result;
}

template<typename T, template <typename, typename> class comparison>
void 
correlation_tool<T, comparison>::set_rectangular_pattern(image<T> &image, point top_left, int width, int height)
{
  //give the size of the template
  pattern_.shape.push_back(point(0,0));
  pattern_.shape.push_back(point(width - 1,height - 1));

  for(int x = 0; x < width; x++)
    for(int y = 0; y < height; y++)
      {
	pattern_.shape.push_back(point(x,y));
      }

  pattern_.values = get_values_to_be_compared(image , top_left ,pattern_.shape); 

}

template<typename T, template <typename, typename> class comparison>
void 
correlation_tool<T, comparison>::set_polygonal_pattern(image<T> &image, point origin, std::vector<point> &polygon)
{
  std::vector<point> polygon_with_boundaries = add_boundaries(polygon);
  std::vector<point>::iterator it = polygon_with_boundaries.begin();

  point top_left = *it;
  it++;
  point bottom_right = *it;

  pattern_.shape.push_back(top_left);
  pattern_.shape.push_back(bottom_right);

  for(int x = top_left.x; x <= bottom_right.x; x++)
    for(int y = top_left.y; y <= bottom_right.y; y++)
      {
	if (point_in_polygon(polygon, point(x,y)))
	  pattern_.shape.push_back(point(x,y));
      }

  //We could have used to the set_pattern function for a slightly shorter code
  //see the code of get_centered_polygonal_pattern 

  pattern_.values = get_values_to_be_compared(image , origin, pattern_.shape); 
}

template<typename T, template <typename, typename> class comparison>
void 
correlation_tool<T, comparison>::set_centered_polygonal_pattern(image<T> &image, point origin, std::vector<point> &polygon)
{
  std::vector<point> polygon_with_boundaries = add_boundaries(polygon);
  std::vector<point>::iterator it = polygon_with_boundaries.begin();
  std::vector<point> polygon_inside_points;

  point top_left = *it;
  it++;
  point bottom_right = *it;

  //find center of gravity
  int center_x = 0;
  int center_y = 0;

  for(std::vector<point>::const_iterator it = polygon.begin(); it != polygon.end(); it++)
    {
      center_x += it->x;
      center_y += it->y;
    }

  center_x /= polygon.size();
  center_y /= polygon.size();

  for(int x = top_left.x; x <= bottom_right.x; x++)
    for(int y = top_left.y; y <= bottom_right.y; y++)
      {
	if (point_in_polygon(polygon, point(x,y)))
	  polygon_inside_points.push_back(point(x - center_x, y - center_y));
      }

  set_pattern(image ,origin, polygon_inside_points); 
}

template<typename T, template <typename, typename> class comparison>
void correlation_tool<T, comparison>::set_centered_rectangular_pattern(image<T> &image, point top_left, int width, int height)
{
  int middle_x = width/2;
  int middle_y = height/2;

  pattern_.shape.push_back(point(0 - middle_x ,0 - middle_y));
  pattern_.shape.push_back(point(width - 1 - middle_x, height - 1 - middle_y));

  for(int x = 0; x < width; x++)
    for(int y = 0; y < height; y++)
      {
	pattern_.shape.push_back(point(x - middle_x, y - middle_y));
      }

  pattern_.values = get_values_to_be_compared(image , top_left ,pattern_.shape); 
}


template<typename T, template <typename, typename> class comparison>
std::vector<point> 
correlation_tool<T,comparison>::add_boundaries(const std::vector<point> &pattern_shape) const
{
  std::vector<point> result;
  std::vector<point>::const_iterator shape_it = pattern_shape.begin();
  
  point upper_left_corner = *shape_it;
  point lower_right_corner = *shape_it;
  result.push_back(*shape_it);//copy the pattern_shape vector
  shape_it++;
  for(;shape_it != pattern_shape.end(); shape_it++)
    {
      result.push_back(*shape_it);//copy the pattern_shape vector
      if(shape_it->x <= upper_left_corner.x)
	upper_left_corner.x = shape_it->x;
      if(shape_it->y <= upper_left_corner.y)
	upper_left_corner.y = shape_it->y;
      if(shape_it->x >= lower_right_corner.x)
	lower_right_corner.x = shape_it->x;	 
      if (shape_it->y >= lower_right_corner.y)
	lower_right_corner.y = shape_it->y;	 
    }
  
  //the order is correct
  result.insert(result.begin(),lower_right_corner);
  result.insert(result.begin(),upper_left_corner);

  return result;
}

template<typename T, template <typename, typename> class comparison>
bool 
correlation_tool<T, comparison>::point_in_polygon(const std::vector<point> &polygon, const point p) const
{
  std::vector<point>::const_iterator i,j;
  bool c = false;
  
  //We say that a point belongs to a polygon 
  //iff its centre belongs to it 
  float x = p.x + .5;
  float y = p.y + .5;
 
  //this code comes from correlation_tracking_tool have a look there for 
  //more details.
  for(i = polygon.begin(), j = --polygon.end() ; i != polygon.end(); j=i,++i)
    {
      if ((((i->y<=y) && (y < j->y)) ||
	   ((j->y<=y) && (y< i->y))) &&
	  (x < (j->x - i->x) * (y - i->y) / (j->y - i->y) + i->x))
	c = !c;
    }

  return c;
}


//the 2 given points describing the boundary are included in the shape
template<typename T, template <typename, typename> class comparison>
std::vector<point> 
correlation_tool<T, comparison>::get_partial_shape(std::vector<point> &shape_description, point top_left, point bottom_right)
{
      std::vector<point> result;
      std::vector<point>::iterator it = shape_description.begin();
      it++; it++; //size information

      //size information
      
      result.push_back(top_left);
      result.push_back(bottom_right);

      for(; it != shape_description.end(); it++)
	{
	  if(it->x >= top_left.x && it->y >= top_left.y  && it->x <= bottom_right.x && it->y <= bottom_right.y)
	    result.push_back(*it);
	}
      return result;
    }


//the first 2 elements of the shape description specify the size of the smallest bounding rectangle covering the shape.
//then it gives relative position to the initial point
//template<typename T>
template<typename T, template <typename, typename> class comparison>
std::vector<T> 
correlation_tool<T, comparison>::get_values_to_be_compared(const image<T> &tracked_image, const point initial_position , const std::vector<point> &shape_description) 
throw (mimasexception)
{ 
  std::vector<T> result;
  
  try{
    //Check the template fits the image, an exception is thrown if not and the function return an empty vector.
    //The calling function then handles the exception to, for instance, get the partial shape.
    std::vector<point>::const_iterator it = shape_description.begin();
    mimas::point top_left(initial_position + *it); 
    it++;
    mimas::point bottom_right(initial_position + *it);
    it++;
        
    //case when we have only part of the template within the image
    if(!(tracked_image.in(top_left) && tracked_image.in(bottom_right)))
      {
	for(;it != shape_description.end(); it++)
	  {
	    if (tracked_image.in(initial_position.x + it->x, initial_position.y + it->y))
	      result.push_back(tracked_image.getPixel(initial_position.x + it->x, initial_position.y + it->y));
	    //else do nothing
	  }	    
	return result; 
      }


    for(;it != shape_description.end(); it++)
      {
	result.push_back(tracked_image.getPixel(initial_position.x + it->x, initial_position.y + it->y));
      }

    return result;
    
  }catch(mimasexception &e)
    {  
      if (e.what() == std::string("Out of image bounds"))
      {
	//send exception to do partial matching
	mimasexception e;
	e << "Get partial pattern shape" ;              
	throw e;  
      }
      else
	std::cerr << "An exception occurred: " <<  e.what() << std::endl;
    }  
}

template<typename T, template <typename, typename> class comparison>
typename correlation_tool<T, comparison>::pattern 
correlation_tool<T, comparison>::get_partial_pattern(const image<T> &image, const point initial_point, const pattern &initial_pattern) 
const
{
  std::vector<point> new_shape;
  std::vector<T> new_values;
  std::vector<point>::const_iterator shape_it = initial_pattern.shape.begin();
  typename std::vector<T>::const_iterator values_it = initial_pattern.values.begin();
  
  //Remember the boundaries of the shape!
  shape_it++;
  shape_it++;

  point upper_left_corner = *shape_it;
  point lower_right_corner = *shape_it;

  for(;shape_it != initial_pattern.shape.end(); shape_it++, values_it++)
    {
      if(image.in(initial_point + *shape_it))
	{
	  //to get the boundaries of the pattern shape
	  if(shape_it->x <= upper_left_corner.x && shape_it->y <= upper_left_corner.y)
	    upper_left_corner = *shape_it;
	  else if(shape_it->x >= lower_right_corner.x && shape_it->y >= lower_right_corner.y)
	    lower_right_corner = *shape_it;
	  
	  new_shape.push_back(*shape_it);
	  new_values.push_back(*values_it);
	}/*do nothing if the point is out of the bound of the image*/
    }

  //the order is correct
  new_shape.insert(new_shape.begin(),lower_right_corner);
  new_shape.insert(new_shape.begin(),upper_left_corner);

  pattern result;
  result.shape = new_shape;
  result.values = new_values;

  return result;
}

//Here instead of using a template we may specify it more as being a binary function or whatever.
//template<typename T, typename comparison>
template<typename T, template <typename, typename> class comparison>
std::vector<double> 
correlation_tool<T,comparison>::correlate(const image<T> &image, std::vector<point> &region_of_interest,
					  pattern &pat, comparison<typename std::vector<T>, typename std::vector<T> > compare) throw(mimasexception) 
//std::binary_function<std::vector<T>, std::vector<T> , double>
{
  std::vector<double> result;
  
  for(std::vector<point>::const_iterator it = region_of_interest.begin(); it != region_of_interest.end(); it++)
    {
      std::vector<T> values = get_values_to_be_compared(image, *it, pat.shape); 
      if(values.size() == pat.values.size())
	result.push_back(compare(pat.values, values));
      else{
	//if the whole pattern is out of the bound of the image i.e. the point of interest is
	//far enough out of the bound of the images such that no point of the 
	//shape pattern are within the bound of the images no correlation measure can be done
	//and therefore we do nothing and continue with next point of the region of interest.
	if(values.size() == 0)
	  continue;
	pattern partial_pattern = get_partial_pattern(image, *it, pat);
	
	std::vector<T> values = get_values_to_be_compared(image, *it, partial_pattern.shape); 
      	result.push_back(compare(partial_pattern.values, values));
      }
    }

  //this is abnormal because if it happens and if time is critical a lot of calculation could have been
  //saved by doing a test on the region of interest before trying to do the correlation.
  //If this situation arised you might think of adding an helper method to the class to do a check
  //according to the region of interest and the shape of your pattern. 
  if (result.size() == 0){
    mimasexception e;
    e << "None of the point of the region of interest bring the shape pattern within the bound of the image";
    throw e;
  }
  return result;
}

template<typename T, template <typename, typename> class comparison>
point
correlation_tool<T,comparison>::get_most_similar_position(const image<T> &image, std::vector<point> &region_of_interest,
							  pattern &pat, comparison<typename std::vector<T>, typename std::vector<T> > compare) throw(mimasexception)
{
  point result;
  //Here we assume that the comparison function gives a higher value for a better similarity 
  if(best_is_max)
    {
      double max = -99999999;
#ifndef NDEBUG
      mimas::timer ti;
#endif
      
      for(std::vector<point>::const_iterator it = region_of_interest.begin(); it != region_of_interest.end(); it++)
	{
	  std::vector<T> values = get_values_to_be_compared(image, *it, pat.shape); 
	  //If the template shape does fit the image only partially
	  if(values.size() != pat.values.size())
	    {
	      //see corresponding comment in correlate function above
	      if(values.size() == 0)
		continue;
	      pattern partial_pattern = get_partial_pattern(image, *it, pat);
	      std::vector<T> values = get_values_to_be_compared(image, *it, partial_pattern.shape); 
	      double tmp = compare(partial_pattern.values, values);
	      if (tmp > max)
		{ 
		  max = tmp;
		  result = *it;
		}
	    }
	  else{
	    double tmp = compare(pat.values, values);
	    if (tmp > max)
	      { 
		max = tmp;
		result = *it;
	      }
	  }
	}
      
      if(max == -99999999)
	{
	  mimasexception e;
	  e<<"None of the point of the region of interest bring the shape pattern within the bound of the image";
	  throw e;
	}
      
#ifndef NDEBUG  
      double time = ti.elapsed("ms");
      std::cerr << "Time to locate the object: " << time <<std::endl;
#endif
      return result;
    }
  else
    {
            double min = 99999999;
#ifndef NDEBUG
      mimas::timer ti;
#endif
      
      for(std::vector<point>::const_iterator it = region_of_interest.begin(); it != region_of_interest.end(); it++)
	{
	  std::vector<T> values = get_values_to_be_compared(image, *it, pat.shape); 
	  //If the template shape does fit the image only partially
	  if(values.size() != pat.values.size())
	    {
	      //see corresponding comment in correlate function above
	      if(values.size() == 0)
		continue;
	      pattern partial_pattern = get_partial_pattern(image, *it, pat);
	      std::vector<T> values = get_values_to_be_compared(image, *it, partial_pattern.shape); 
	      double tmp = compare(partial_pattern.values, values);
	      if (tmp < min)
		{ 
		  min = tmp;
		  result = *it;
		}
	    }
	  else{
	    double tmp = compare(pat.values, values);
	    if (tmp < min)
	      { 
		min = tmp;
		result = *it;
	      }
	  }
	}
      
      if(min == 99999999)
	{
	  mimasexception e;
	  e<<"None of the point of the region of interest bring the shape pattern within the bound of the image";
	  throw e;
	}
      
#ifndef NDEBUG  
      double time = ti.elapsed("ms");
      std::cerr << "Time to locate the object: " << time <<std::endl;
#endif
      return result;
    }
}


template<typename T, template <typename, typename> class comparison>
void 
correlation_tool<T,comparison>::set_pattern(const image<T> &image, const point origin, const std::vector<point> &pattern_shape) 
throw (mimasexception)
{
  pattern_.shape = add_boundaries(pattern_shape);
  try{
    pattern_.values = get_values_to_be_compared(image, origin, pattern_.shape);
  }
  catch(mimasexception &e)
    {
      mimasexception another_one;
      another_one <<"The pattern you tried to instanciate try to get its values out of the bound of the image";
      throw another_one;
    }
}
  
