/**********************************************************************
*example code how to use the hypothesis filter.
*
*Here is a hack, we re-use the code of the hypothesis filter
*to correlate the whole image with the model image.
*It exhibits that parts of the image that are not representing
*the pipette looks similar to it when we correlate them. (clutter)
*
*
* @author: Manuel Boissenin
* @date: 20/05/2004
*
* Time-stamp: <2005-05-20 13:52:50 engmb>
* Copyright (C) 2004 by Manuel Boissenin
*
 ***********************************************************************/

#include <string>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <image.h>
#include <image_fileinput.h>
#include <image_fileoutput.h>
#include <GL/glut.h>
#include <image_mesaoutput.h>
#include "condensation.h"
#include "hypothesis2D.h"
#include "correlation_observer2D.h"
#include "analyse_result_mean.h"
#include "image_loader.h"

using namespace mimas::hf;
using namespace std;

int main(int argc, char** argv)
{
  int retVal = 0;
  try {
    glutInit(&argc, argv);
    mimas::x11_display disp; 
    image_mesaoutput< unsigned char > display(&disp);
    
    hypothesis2D init(atoi(argv[2]),atoi(argv[3]),1.);
    hypothesis2D::sigmaSpan = atof(argv[4]);

    //parametres for the different part of the host class.
    image_loader<unsigned char>::set il_param(argv[1], 99, ".png");
    correlation_observer2D<hypothesis2D, unsigned char>::set obs_param(atoi(argv[2]),atoi(argv[3]), 40, 40);
    analyse_result_mean<hypothesis2D, unsigned char>::set analyse_param;
    
    hypothesis_filter<image_loader, correlation_observer2D, analyse_result_mean, hypothesis2D>
      hf(init, atoi(argv[5]), il_param, obs_param, analyse_param);

    int imageNumber = 0;

    image<unsigned char> resultImage;
    resultImage.init(hf.image.getWidth(), hf.image.getHeight());
    while(true)
      {
	try{hf.next();}
	catch (mimasexception &e){
	  std::cerr<<e.what()<<std::endl;  
	  return 0;
	}
	
	for (int i = 40; i < hf.image.getWidth() - 40; i++)
	  for (int j = 40; j < hf.image.getHeight() - 40 ; j++)
	    {
	      double res = hf.convolvePoint(i,j);
	      resultImage.setPixel(i,j, (unsigned char)((res + 1) * 127.49));
	    }
	

	//----------save on a file--------//
	
	std::ostringstream str;
	str << "results/correlatedPipette" << std::setw(3) << std::setfill( '0' ) <<  imageNumber++
	    << ".ppm";
	std::ofstream filteredOut(str.str().c_str(), std::ios::binary); 
	filteredOut << resultImage;
	//---------------------------------//
	
	display << resultImage;//<< mimas::pause();
      }
	
    
  } catch ( exception &e )
    {
      cerr << "An exception occurred: " <<  e.what() << endl;
      retVal = 1;
    };
  return retVal;
}
