/**********************************************************************
*example code how to use the hypothesis filter.
*
*Here is an implementation that seek to use the hypothesis filter
*to do recognition of the object in the image.
*It assumes the continuity of the correlation measure. So bad measures
*indicate that in the object is not in the neighborhood and good
*measures are an incentive to explore thoroughly a neighborhood.
*Feel free to publish this idea as long as you cite me.
*
* @author: Manuel Boissenin
* @date: 20/05/2004
*
* Time-stamp: <2005-05-23 20:15:00 engmb>
* Copyright (C) 2004 by Manuel Boissenin
*
 ***********************************************************************/

#include <string>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <image.h>
#include <image_fileinput.h>
#include <image_fileoutput.h>
#include <GL/glut.h>
#include <image_mesaoutput.h>

//!!change the name of p_condensation

#include "p_condensation.h"
#include "hypothesis2D.h"
#include "correlation_observer2D.h"
#include "analyse_result_mean.h"
#include "image_loader.h"
#include "compensate_speed.h"

using namespace mimas::hf;
using namespace std;

class PDE{  //for probability density element
public:
  float pbToBeChosen;
  bool measured;
  float measuredValue;
  
  PDE(float initialPb = .5)
    :pbToBeChosen(initialPb), measured(false){}
};

int abs(int i)
{
  if (i< 0)
    return -i;
  return i;
}
  
void changePdf(image<PDE>::iterator &it)
{
  if ((*it).measuredValue <= 0)
    {
      for(int i = -5; i <= 5; i++)            //not a problem here as values are far from edges
	for(int j = -5  -abs(i/2); j <= 5 + abs (i/2); j++)           //to have a shape more like a "circle"
	  {
	    dist2D d(i,j);
	    image<PDE>::iterator temp = it + d;
	    (*temp).pbToBeChosen = 0;	    
	  }
    }
  else if ((*it).measuredValue <= .2)
    {
      for(int i = -3; i <= 3; i++)            
	for(int j = -3; j <= 3; j++)           
	  {
	    dist2D d(i,j);
	    image<PDE>::iterator temp = it + d;
	    (*temp).pbToBeChosen = .1;	    
	  }
    }
  else if ((*it).measuredValue <= .4)
    {
      for(int i = -4; i <= 4; i++)            
	for(int j = -4; j <= 4; j++)           
	  {
	    dist2D d(i,j);
	    image<PDE>::iterator temp = it + d;
	    (*temp).pbToBeChosen = .8;	    
	  }
    }
  else 
    {
      for(int i = -5; i <= 5; i++)            
	for(int j = -5; j <= 5; j++)           
	  {
	    dist2D d(i,j);
	    image<PDE>::iterator temp = it + d;
	    (*temp).pbToBeChosen = 1.;	    
	  }

    }

}

int main(int argc, char** argv)
{
  int retVal = 0;
  try {
    glutInit(&argc, argv);
    mimas::x11_display disp; 
    image_mesaoutput< unsigned char > display(&disp);

    image<PDE> pdf;
    
    image_loader<unsigned char>::set il_param(argv[1], 99, ".png");
    correlation_observer2D<hypothesis2D, unsigned char>::set obs_param(atoi(argv[2]), atoi(argv[3]), 40, 40);

    //image_loader<unsigned char> il(image, il_param);
    //    correlation_observer2D<hypothesis2D, unsigned char > obs(image, obs_param);

    analyse_result_mean<hypothesis2D, unsigned char>::set analyse_param;

    hypothesis2D init(100, 100 ,1.); //doesn't matter as we are not tracking
    hypothesis2D::sigmaSpan = 20;
    
    hypothesis_filter<image_loader, correlation_observer2D, analyse_result_mean, hypothesis2D>
      hf(init, atoi(argv[4]), il_param, obs_param, analyse_param);
    
    int NbHypothesis = atoi(argv[4]);
    
    std::vector<hypothesis2D> hypotheses;
    std::vector<hypothesis2D> bestHypotheses;

    //hack to change image
    for(int i = 0; i<34; i++)
      {
	hf.next();
      }

    pdf.init(hf.image.getWidth(), hf.image.getHeight());
    
    for(int i = 0; i < NbHypothesis; i++)
      {
	//Choose an hypothesis
	hypothesis2D hypothesis( (int)( 40 + (hf.image.getWidth() - 80)  * Dice::uniform_random()) , (int)( 40 + (hf.image.getHeight() -80)  * Dice::uniform_random()));

	image<PDE>::iterator it = pdf.ul();

	//	std::cerr<<"x:  "<< hypothesis.x <<std::endl;
	//std::cerr<<"y:  "<< hypothesis.y <<std::endl;
	it.x += int(hypothesis.x);
	it.y += int(hypothesis.y);

	//std::cerr<<"initial prob "<< (*it).pbToBeChosen <<std::endl;
	//PDE &pde = getPixel(hypothesis.x , hypothesis.y);

	if(!(*it).measured && ( (*it).pbToBeChosen >  Dice::uniform_random() ))
	{
	   hf.observe(hypothesis);
	   std::cerr <<"Weight is: "<< hypothesis.weight << std::endl;
	   (*it).measuredValue = hypothesis.weight;
	   (*it).measured = true;
	   changePdf(it);
	}

      }
	   
	display << hf.image<< mimas::pause();
    /*
    //distribute randomly over the image the hypotheses
    for(int i = 0; i < NbHypothesis; i++)
      {
	hypotheses.push_back( hypothesis2D( (int)(hf.image.getWidth()  * Dice::uniform_random()) , (int)(hf.image.getHeight() * Dice::uniform_random())) );
	std::cerr <<"x position :"<< hypotheses[0].x << std::endl;
      }

    bool first = true;
    
    while(true) ///////!!!
      {
	//take measures
	for(std::vector<hypothesis2D>::iterator it_h = hypotheses.begin(); it_h != hypotheses.end(); it_h++)
	  {
	    hf.observe(*it_h);
	    std::cerr <<"Weight is: "<< it_h->weight << std::endl;
	  }
      
    //select best locations, first time we need to find the max then, we can just consider the best previous location 
    //(the best ones and the other that are 90 percent of the weight of the best one   (and save it))

	double factor = .9;
	
	if (first)
	  {
	    double max_weight = 0;
	    for(std::vector<hypothesis2D>::iterator it_h = hypotheses.begin(); it_h != hypotheses.end(); it_h++)
	      {
		if (it_h->weight > max_weight)
		  {
		    max_weight = it_h->weight;
		  }		  
	      }

	    for(std::vector<hypothesis2D>::iterator it_h = hypotheses.begin(); it_h != hypotheses.end(); it_h++)
	      if (it_h->weight > factor * max_weight)
		bestHypotheses.push_back(*it_h);
	      
	    first = false;
	  }
	else{


	}
    //redistribute a percentage around these best points and the rest randomly 

    //display on an image the best locations and the hypothesis

	for(std::vector<hypothesis2D>::iterator it_h = bestHypotheses.begin(); it_h != bestHypotheses.end(); it_h++)
	  hf.draw(*it_h);
	
	display << hf.image;//<< mimas::pause();

	      

      }
    */
      
    
  } catch ( exception &e )
    {
      cerr << "An exception occurred: " <<  e.what() << endl;
      retVal = 1;
    };
  return retVal;
}
