/**********************************************************************
*example code how to use the hypothesis filter.
*
*
* @author: Manuel Boissenin
* @date: 20/05/2004
*
* Time-stamp: <2005-05-11 14:53:56 engmb>
* Copyright (C) 2004 by Manuel Boissenin
*
 ***********************************************************************/
#include <string>
#include <fstream>
#include <iostream>
#include <iomanip>
#include <sstream>

//#include <x11_display.h>
#include <mesa_offscreen.h>

#include <image.h>
#include <rgba.h>
#include <image_fileinput.h>
#include <image_fileoutput.h>
#include <image_mesaoutput.h>

#include <analyse_result_blob.h>
#include <hypothesis3D.h>
#include <image_loader.h>
#include <p_condensation.h>

#include "openGL_observer.h"

using namespace mimas::hf;

int main(int argc, char** argv)
{
  int retVal = 0;
  try {

    //Initialisation tracker
    x11_display *dpy = new x11_display;

    //glutInit(&argc, argv);
    hypothesis3D init(atof(argv[2]),atof(argv[3]),atof(argv[4]),atof(argv[5]),atof(argv[6]),atof(argv[7]));
  
    hypothesis3D::transX = atof(argv[8]);
    hypothesis3D::transY = atof(argv[9]);
    hypothesis3D::transZ = atof(argv[10]);
    hypothesis3D::rotX = atof(argv[11]);
    hypothesis3D::rotY = atof(argv[12]);
    hypothesis3D::rotZ = atof(argv[13]);

    image_loader<rgba<unsigned char> >::set il_param(argv[1],99,".png",2);;
    openGL_observer<hypothesis3D, rgba<unsigned char> >::set obs_param(dpy);
    analyse_result_blob<hypothesis3D, rgba<unsigned char> >::set analyse_param(.35, 60, 45);
  
    hypothesis_filter<image_loader, openGL_observer, analyse_result_blob, hypothesis3D,
      p_condensation, no_compensation, mimas::rgba<unsigned char> > hf(init, atoi(argv[14]), il_param, obs_param, analyse_param);

    //tracking
    //    image_mesaoutput<rgba<unsigned char> > show(dpy, hf.image.getWidth(), hf.image.getHeight());

    //!!!load the distance image, better to calculate it live for a real-time demonstration

    std::string distance_image_basename = "Filtered/Filtered";
    //  distance_image_basename += "dist";
    
    image_loader<int > distance_map
      (hf.filteredImage,
       image_loader<int >::set(distance_image_basename, 99, ".ppm", 3, 0));


    std::string fileOutputBaseName = "image_result";
    int imageNumber = 0;
    

    while(true)
      {
	
	try{hf.next();
	distance_map.next();
	}
	catch (mimasexception &e){
	  std::cerr<<e.what()<<std::endl;  
	  return 0;
	}

	std::ostringstream str;
	str << fileOutputBaseName << std::setw(3) << std::setfill( '0' ) <<  imageNumber++
	    << ".ppm";
	
	std::ofstream fileOut(str.str().c_str(), std::ios::binary); 
	
	std::vector<hypothesis3D> estimatedLocation;
	//filter the image and instanciate it for the observer

	//load the distance map to edges of the image.

	//hf.filteredImage = gradient(hf.image.ul(), hf.image.lr(),40);
	
	//show << hf.image<< mimas::delay(500);
	//show << hf.filteredImage<< mimas::delay(500);
	hf.filter();
	//show << hf.offscreenImage<<mimas::delay(500);
	
	estimatedLocation = hf.analyse();

	rgba<unsigned char> col(0,255,255);
	hf.draw(estimatedLocation, col);     //from Observer	

	//find the best peak and do a partial reinitialisation around it
	std::vector<hypothesis3D>::iterator it_h, it_h_max;
	float temp_weight = 0;
	
	for(it_h = estimatedLocation.begin(); it_h != estimatedLocation.end(); it_h++)
	  if (it_h->weight > temp_weight)
	    {
	      temp_weight = it_h->weight;
	      it_h_max = it_h;
	    }
	///if an hypothesis has an excellent result we did a partial_reset around it

	for(it_h = hf.hypotheses.begin(); it_h != hf.hypotheses.end(); it_h++)
	  if (it_h->weight > .85)
	    {
	      hf.partial_reset(*it_h, .05, 0, .1, true);
	      std::cerr<<"One partial reinitialisation done" <<std::endl;
	    }

	
	//Draw the information on the image
	if(temp_weight != 0)
	  {
	    rgba<unsigned char> red(255, 0, 0);
	    //hf.partial_reset(*it_h_max, .04, .2);
	    std::cerr<< "Max weight is: "<< temp_weight << std::endl;
	    std::cerr<< "For hypothesis: "<< std::endl;
	    hf.draw(*it_h_max, red);

	    ///Write result on the screen

	    mesa_offscreen<mimas::rgba<unsigned char> > m(dpy, hf.image, false);
	    m.ortho2D();
	    m.setColor(red);
	    
	    glPushMatrix();
	    glTranslated(10, 15, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
	    std::ostringstream str2;
	    str2 << "X: "<< (*it_h_max).x;
	    m.text(str2.str());
	    glPopMatrix();
  
  
	    glPushMatrix();
	    glTranslated(10, 30, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
	    std::ostringstream str3;
	    str3  << "Y: " << (*it_h_max).y ;
	    m.text(str3.str());
	    glPopMatrix();
	    
	    
	    glPushMatrix();
	    glTranslated(10, 45, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
	    std::ostringstream str4;
	    str4 <<  "Z: " <<  (*it_h_max).z; 
	    m.text(str4.str());
	    glPopMatrix();

	    
	    glPushMatrix();
	    glTranslated(10, 60, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
	    std::ostringstream str5;
	    str5 << "Rx: "<< (*it_h_max).Rx;
	    m.text(str5.str());
	    glPopMatrix();

  
	    glPushMatrix();
	    glTranslated(10, 75, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
  	    std::ostringstream str6;
	    str6 <<"Ry: "<< (*it_h_max).Ry ;
  	    m.text(str6.str());
	    glPopMatrix();

  
	    glPushMatrix();
	    glTranslated(10, 90, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
    	    std::ostringstream str7;
	    str7 << "Rz: "<< (*it_h_max).Rz;
  	    m.text(str7.str());
	    glPopMatrix();
  
	    m.finish();
	    
	  }

	fileOut << hf.image;

      }

  } catch ( std::exception &e ) {
    std::cerr << e.what() << std::endl;
    retVal = 1;
  };
  return retVal;
}





