#ifndef IMAGE_DRAW_H
#define IMAGE_DRAW_H

#include <cmath>
#include "image.h"

namespace mimas{

/* Bresenham line function */
template< typename T>
  void drawLine( image_ref<T> &image, int x1, int y1, int x2, int y2, T colour = T())
{
  int i,dx,dy,sdx,sdy,dxabs,dyabs,x,y,px,py;

  dx=x2-x1;			/* the horizontal distance of the line */
  dy=y2-y1;			/* the vertical distance of the line */
  dxabs=std::abs(dx);
  dyabs=std::abs(dy);
  sdx=sgn(dx);
  sdy=sgn(dy);
  x=dyabs>>1;
  y=dxabs>>1;
  px=x1;
  py=y1;

  image.setPixel(px,py,colour);

  if (dxabs>=dyabs) /* the line is more horizontal than vertical */
    {
      for(i=0;i<dxabs;i++)
	{
	  y+=dyabs;
	  if (y>=dxabs)
	    {
	      y-=dxabs;
	      py+=sdy;
	    }
	  px+=sdx;
	  image.setPixel(px,py,colour);
	}
    }
  else /* the line is more vertical than horizontal */
    {
      for(i=0;i<dyabs;i++)
	{
	  x+=dxabs;
	  if (x>=dyabs)
	    {
	      x-=dyabs;
	      px+=sdx;
	    }
	  py+=sdy;
	  image.setPixel(px,py,colour);


	}
    }
}
		
template< typename T>
void drawBox(image_ref<T> &image, int x1, int y1, int x2, int y2, T colour = T())
{
  drawLine(image, x1,y1,x1,y2, colour);
  drawLine(image, x1,y1,x2,y1, colour);
  drawLine(image, x2,y1,x2,y2, colour);
  drawLine(image, x1,y2,x2,y2, colour);
}

template< typename T>
void drawThickLine(image_ref<T> &image, int x1, int y1, int x2, int y2, T colour, int width)
{
  int i,dx,dy,sdx,sdy,dxabs,dyabs,x,y,px,py;
  int p,q;

  dx=x2-x1;			/* the horizontal distance of the line */
  dy=y2-y1;			/* the vertical distance of the line */
  dxabs=std::abs(dx);
  dyabs=std::abs(dy);
  sdx=sgn(dx);
  sdy=sgn(dy);
  x=dyabs>>1;
  y=dxabs>>1;
  px=x1;
  py=y1;

  for (p=-width/2; p<width/2; p++)
    for (q=-width/2; q<width/2; q++)
      image.setPixel(px+p,py+q,colour);

  if (dxabs>=dyabs) /* the line is more horizontal than vertical */
    {				for(i=0;i<dxabs;i++)
      {
	y+=dyabs;
	if (y>=dxabs)
	  {
	    y-=dxabs;
	    py+=sdy;
	  }
	px+=sdx;

	for (p=-width/2; p<width/2; p++)
	  for (q=-width/2; q<width/2; q++)
	    image.setPixel(px+p,py+q,colour);
      }
    }
  else /* the line is more vertical than horizontal */
    {
      for(i=0;i<dyabs;i++)
	{
	  x+=dxabs;
	  if (x>=dyabs)
	    {
	      x-=dyabs;
	      px+=sdx;
	    }
	  py+=sdy;
					
	  for (p=-width/2; p<width/2; p++)
	    for (q=-width/2; q<width/2; q++)
	      image.setPixel(px+p,py+q,colour);

	}
    }
}

}

#endif
