#ifndef IMAGE_XINEINPUT_H
#define IMAGE_XINEINPUT_H

#include "mimasconfig.h"

#ifdef HAVE_XINE

#ifdef HAVE_CONFIG_H
#undef HAVE_CONFIG_H
#include <xine.h>
#include <xine/xineutils.h>
#define HAVE_CONFIG_H
#else
#include <xine.h>
#include <xine/xineutils.h>
#endif
#include "image_input.h"
#include "rgba.h"

namespace mimas {

/** @addtogroup imageIO
    @{ */
/** @addtogroup imageInput
    @{ */
/** Class for reading images from a media resource location.
    The specified media resource location is opened with libxine.
    This class is using <A HREF="http://www.xinehq.de/">xine</A>
    to capture frames from a video source.
    
    Note, that this is not an efficient way to do it, if you just want to
    display movies. The images are converted to RGB and for displaying,
    they need to undergo another colorspace conversion.
    
    At the moment, mimas does not support output to a video file. You can
    use <B>mencoder</B> for this.
    \verbatim
    mencoder tv:// -tv driver=v4l:device=/dev/video0:width=640:height=480:norm=PAL:channel=1:input=1 -ovc lavc -nosound -lavcopts vcodec=msmpeg4:vbitrate=4000
    \endverbatim

    Grabbing frames from a running video is done like this:
    \include viewVideo/minimal.cc

    @author Jan Wedekind (jan@wedesoft.de)
    @date Fri Jan 06 21:22:51 UTC 2006 */
template< typename T >
class image_xineinput: public image_input< T >
{
 public:
  /** Constructor.
      The constructor takes a media resource location as argument.
      @param _mrl Media resource location.
      @param vo_driver Video output driver. You can specify \"auto\" here.
      @param ao_driver Audio output driver. You can specify \"auto", if you
      want audio output. */
  explicit image_xineinput( const std::string &_mrl,
                            const char *vo_driver = "auto",
                            const char *ao_driver = NULL )
    throw (mimasexception);
  ///
  virtual ~image_xineinput(void);
  /** Read image from media resource.
      @param img Object to store image in. */
  virtual void read( image< T > &img ) throw (mimasexception);
 protected:
  /// Media resource location.
  std::string mrl;
  /// Instance of xine-player.
  xine_t *xine;
  /// Video output driver instance.
  xine_video_port_t *vo_port;
  /// Audio output driver instance.
  xine_audio_port_t *ao_port;
  /// Video stream instance.
  xine_stream_t *stream;
  ///
  int width;
  ///
  int height;
  ///
  int format;
};

///@}

///@}

};

#include "image_xineinput.tcc"

#endif

#endif
